import sqlite3
import csv
from datetime import datetime, date
from gtts import gTTS
from pygame import mixer
import os


def connect():
    """Returns a connection object"""
    return sqlite3.connect("valid_persons.db")


class Attendance:
    # Returns True if already marked and False if not marked
    @staticmethod
    def if_marked(face_encoding):
        con = connect()
        cur = con.cursor()
        cur.execute(
            "SELECT * FROM attendance WHERE date = ? AND person_id = (SELECT id FROM persons WHERE face_encoding = ?)",
            (datetime.today().strftime("%Y-%m-%d"), face_encoding.tobytes()),
        )
        if not cur.fetchone():
            return False
        else:
            return True

    # mark attendance of a detected person
    @staticmethod
    def mark_attendance(face_encodings: list):
        con = connect()
        cur = con.cursor()
        if face_encodings:
            for encoding in face_encodings:
                if Attendance.if_marked(encoding) == False:
                    cur.execute(
                        "SELECT id,name from persons WHERE face_encoding = ?",
                        (encoding.tobytes(),),
                    )
                    result = cur.fetchone()
                    name = result[1]
                    person_id = result[0]
                    cur.execute(
                        "INSERT INTO attendance(date,person_id) VALUES(?,?)",
                        (datetime.today().strftime("%Y-%m-%d"), person_id),
                    )
                    con.commit()
                    Attendance.give_feedback(f"{name} your attendance has been noted")

    # generate a csv with the attendance for a particular date
    @staticmethod
    def generate_csv_date(date_for_attendance):
        try:
            date.fromisoformat(date_for_attendance)
        except ValueError:
            return "Wrong date format!"
        con = connect()
        cur = con.cursor()
        cur.execute(
            "SELECT name,emp_id FROM persons WHERE id IN(SELECT person_id FROM attendance WHERE date = ?)",
            (date_for_attendance,),
        )
        results = cur.fetchall()
        if not results:
            return f"No data available for {date_for_attendance}"
        os.makedirs("csv/", exist_ok=True)
        with open(f"csv/{date_for_attendance}.csv", "w") as csvfile:
            writer = csv.writer(csvfile)
            writer.writerow(["Name", "Emp_id"])
            for result in results:
                writer.writerow(result)
        return f"CSV file generated!"

    @staticmethod
    def generate_csv_emp(emp_id):
        emp_id = str(emp_id)
        con = connect()
        cur = con.cursor()
        cur.execute("SELECT * FROM persons WHERE emp_id =?", (emp_id,))
        if not cur.fetchone():
            return f"No person with emp id {emp_id} exists"
        cur.execute(
            "SELECT date FROM attendance WHERE person_id = (SELECT id FROM persons WHERE emp_id = ?)",
            (emp_id,),
        )
        results = cur.fetchall()
        if not results:
            return f"No data available for employee with id {emp_id}"
        os.makedirs("csv/", exist_ok=True)
        with open(f"csv/{emp_id}.csv", "w") as csvfile:
            writer = csv.writer(csvfile)
            writer.writerow(["Date"])
            for result in results:
                writer.writerow(result)
        return f"CSV file generated!"

    # play the feedback sound
    @staticmethod
    def give_feedback(text):
        tts = gTTS(text)
        tts.save("feedback.mp3")
        mixer.init()
        sound = mixer.Sound("feedback.mp3")
        sound.play()
        os.remove("feedback.mp3")



import face_recognition
import cv2
import numpy as np
from faceattend.face import Face
from faceattend.attendance import Attendance


def detect_face():

    # Get the valid face data from database
    known_face_encodings, known_face_names = Face.load_faces()

    # Initialize variables
    face_names = []
    face_locations = []
    face_encodings = []
    process_this_frame = True

    # start video capture
    video_capture = cv2.VideoCapture(0)

    while True:
        # Grab a single frame of video
        _, frame = video_capture.read()

        # Only process every other frame of video to save time
        if process_this_frame:
            # Resize frame of video to 1/4 size for faster face recognition processing
            small_frame = cv2.resize(frame, (0, 0), fx=0.25, fy=0.25)

            # Convert the image from BGR color (which OpenCV uses) to RGB color (which face_recognition uses)
            rgb_small_frame = cv2.cvtColor(small_frame, cv2.COLOR_BGR2RGB)

            # Find all the faces and face encodings in the current frame of video
            face_locations = face_recognition.face_locations(rgb_small_frame)
            face_encodings = face_recognition.face_encodings(
                rgb_small_frame, face_locations, 1
            )

            face_names = []
            attendance_face_encodings = []
            for face_encoding in face_encodings:
                # See if the face is a match for the known face(s)
                matches = face_recognition.compare_faces(
                    known_face_encodings, face_encoding, tolerance=0.5
                )
                name = "Stranger"

                # the known face with the smallest distance to the new face
                face_distances = face_recognition.face_distance(
                    known_face_encodings, face_encoding
                )
                best_match_index = np.argmin(face_distances)
                if matches[best_match_index]:
                    name = known_face_names[best_match_index]
                    encoding = known_face_encodings[best_match_index]
                if name in face_names:
                    continue
                face_names.append(name)
                if name != "Stranger":
                    attendance_face_encodings.append(encoding)

        Attendance.mark_attendance(attendance_face_encodings)
        process_this_frame = not process_this_frame
        # Display the results
        for (top, right, bottom, left), name in zip(face_locations, face_names):
            # Scale back up face locations since the frame we detected in was scaled to 1/4 size
            top *= 4
            right *= 4
            bottom *= 4
            left *= 4

            # Draw a red box around the unknown face and green box around the known face
            if name == "Stranger":
                cv2.rectangle(frame, (left, top), (right, bottom), (0, 0, 255), 2)

                # Draw a label  below the face
                cv2.rectangle(
                    frame, (left, bottom - 35), (right, bottom), (0, 0, 255), cv2.FILLED
                )
            else:
                cv2.rectangle(frame, (left, top), (right, bottom), (0, 255, 0), 2)

                cv2.rectangle(
                    frame, (left, bottom - 35), (right, bottom), (0, 255, 0), cv2.FILLED
                )
            # Add the name to the label
            font = cv2.FONT_HERSHEY_DUPLEX
            cv2.putText(frame, name, (left + 6, bottom - 6), font, 1.0, (0, 0, 0), 1)

        # Display the resulting image
        cv2.imshow("Video", frame)

        # Hit 'q' on the keyboard to quit!
        if cv2.waitKey(1) & 0xFF == ord("q"):
            break

    # Release handle to the webcam
    video_capture.release()
    cv2.destroyAllWindows()


if __name__ == "__main__":
    detect_face()



from face_recognition import (
    face_encodings,
    load_image_file,
    face_locations,
)
import sqlite3
import numpy as np
import random


# generate a random 10 digit number for emp id
def generate_emp_id():
    return str(random.randint(1000000000, 9999999999))


def connect():
    """Returns a connection object"""
    return sqlite3.connect("valid_persons.db")


class Face:
    # add the face encoding bytes to the database for the specific person
    @staticmethod
    def add_face(name, img_file):
        if not img_file:
            return "An image file is required!"
        if not name:
            return "Name is required!"
        try:
            img = load_image_file(img_file)
        except FileNotFoundError:
            return f"Wrong Path or file name! No such file as '{img_file}'"
        faceloc = face_locations(img)
        face_encode = face_encodings(img, faceloc, 1)[0]
        emp_id = generate_emp_id()
        con = connect()
        cur = con.cursor()
        try:
            cur.execute(
                "INSERT INTO persons (name,emp_id,face_encoding) VALUES(?,?,?)",
                (name, emp_id, face_encode.tobytes()),
            )
            con.commit()
            con.close()
            return f"Face added for {name} with employee id {emp_id}"
        except sqlite3.IntegrityError:
            return "Face already exists!"

    # Delete the person from database given its emp id
    @staticmethod
    def delete_face(emp_id):
        con = connect()
        cur = con.cursor()
        cur.execute("SELECT * FROM persons WHERE emp_id = ?", (emp_id,))
        if not cur.fetchone():
            return f"No person with emp id of {emp_id} was found!"
        cur.execute("DELETE FROM persons WHERE emp_id = ?", (emp_id,))
        con.commit()
        return f"Person with {emp_id} has been removed!"

    # read all the face_encodings and names from the database and return and list containg all the known face_encodings and name
    @staticmethod
    def load_faces():
        con = connect()
        cur = con.cursor()
        cur.execute("SELECT name,face_encoding FROM persons")
        encodings = cur.fetchall()
        known_face_encodings = [encoding[1] for encoding in encodings]
        known_face_names = [name[0] for name in encodings]
        known_face_encodings = [
            np.frombuffer(encoding, dtype="float64")
            for encoding in known_face_encodings
        ]
        con.close()
        return known_face_encodings, known_face_names



import sqlite3


def create_database():
    conn = sqlite3.connect("valid_persons.db")
    conn.cursor().execute(
        """CREATE TABLE IF NOT EXISTS persons(
    id INTEGER PRIMARY KEY,
    name TEXT NOT NULL,
    emp_id TEXT NOT NULL,
    face_encoding BLOB UNIQUE NOT NULL
    )"""
    )
    conn.cursor().execute(
        """CREATE TABLE IF NOT EXISTS attendance(
    id INTEGER PRIMARY KEY,
    date DATE NOT NULL,
    person_id INTEGER NOT NULL,
    FOREIGN KEY (person_id) REFERENCES persons(id)
    )"""
    )
    conn.close()
    return "Database created!"


if __name__ == "__main__":
    create_database()



from setuptools import setup

from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="faceattend",
    version="0.0.4",
    description="An attendance system using face recognition",
    url="https://github.com/AkashMondal1998/FaceAttend",
    author="Akash Mondal",
    author_email="mondalakash63@gmail.com",
    license="MIT license",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=["faceattend"],
    install_requires=["face-recognition", "gTTS", "pygame", "opencv-python"],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
    ],
)
